@@ -16,6 +16,7 @@
 import logging
 
 import json
+import ast
 from webob import Response
 
 from ryu.base import app_manager
@@ -155,7 +156,7 @@ class StatsController(ControllerBase):
             flow = {}
         else:
             try:
-                flow = eval(req.body)
+                flow = ast.literal_eval(req.body)
             except SyntaxError:
                 LOG.debug('invalid syntax %s', req.body)
                 return Response(status=400)
@@ -258,7 +259,7 @@ class StatsController(ControllerBase):
             groups = ofctl_v1_2.get_group_features(dp, self.waiters)
         elif dp.ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
             groups = ofctl_v1_3.get_group_features(dp, self.waiters)
-        elif do.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
+        elif dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
             LOG.debug('Request not supported in this OF protocol version')
             return Response(status=501)
         else:
@@ -277,7 +278,7 @@ class StatsController(ControllerBase):
             groups = ofctl_v1_2.get_group_desc(dp, self.waiters)
         elif dp.ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
             groups = ofctl_v1_3.get_group_desc(dp, self.waiters)
-        elif do.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
+        elif dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
             LOG.debug('Request not supported in this OF protocol version')
             return Response(status=501)
         else:
@@ -296,7 +297,7 @@ class StatsController(ControllerBase):
             groups = ofctl_v1_2.get_group_stats(dp, self.waiters)
         elif dp.ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
             groups = ofctl_v1_3.get_group_stats(dp, self.waiters)
-        elif do.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
+        elif dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
             LOG.debug('Request not supported in this OF protocol version')
             return Response(status=501)
         else:
@@ -326,7 +327,7 @@ class StatsController(ControllerBase):
 
     def mod_flow_entry(self, req, cmd, **_kwargs):
         try:
-            flow = eval(req.body)
+            flow = ast.literal_eval(req.body)
         except SyntaxError:
             LOG.debug('invalid syntax %s', req.body)
             return Response(status=400)
@@ -366,12 +367,14 @@ class StatsController(ControllerBase):
         if dp is None:
             return Response(status=404)
 
+        flow = {'table_id': dp.ofproto.OFPTT_ALL}
+
         if dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
             ofctl_v1_0.delete_flow_entry(dp)
         elif dp.ofproto.OFP_VERSION == ofproto_v1_2.OFP_VERSION:
-            ofctl_v1_2.mod_flow_entry(dp, {}, dp.ofproto.OFPFC_DELETE)
+            ofctl_v1_2.mod_flow_entry(dp, flow, dp.ofproto.OFPFC_DELETE)
         elif dp.ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
-            ofctl_v1_3.mod_flow_entry(dp, {}, dp.ofproto.OFPFC_DELETE)
+            ofctl_v1_3.mod_flow_entry(dp, flow, dp.ofproto.OFPFC_DELETE)
         else:
             LOG.debug('Unsupported OF protocol')
             return Response(status=501)
@@ -380,7 +383,7 @@ class StatsController(ControllerBase):
 
     def mod_meter_entry(self, req, cmd, **_kwargs):
         try:
-            flow = eval(req.body)
+            flow = ast.literal_eval(req.body)
         except SyntaxError:
             LOG.debug('invalid syntax %s', req.body)
             return Response(status=400)
@@ -413,7 +416,7 @@ class StatsController(ControllerBase):
 
     def mod_group_entry(self, req, cmd, **_kwargs):
         try:
-            group = eval(req.body)
+            group = ast.literal_eval(req.body)
         except SyntaxError:
             LOG.debug('invalid syntax %s', req.body)
             return Response(status=400)
@@ -448,7 +451,7 @@ class StatsController(ControllerBase):
 
     def mod_port_behavior(self, req, cmd, **_kwargs):
         try:
-            port_config = eval(req.body)
+            port_config = ast.literal_eval(req.body)
         except SyntaxError:
             LOG.debug('invalid syntax %s', req.body)
             return Response(status=400)
@@ -493,7 +496,7 @@ class StatsController(ControllerBase):
             return Response(status=404)
 
         try:
-            exp = eval(req.body)
+            exp = ast.literal_eval(req.body)
         except SyntaxError:
             LOG.debug('invalid syntax %s', req.body)
             return Response(status=400)
@@ -502,7 +505,7 @@ class StatsController(ControllerBase):
             ofctl_v1_2.send_experimenter(dp, exp)
         elif dp.ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
             ofctl_v1_3.send_experimenter(dp, exp)
-        elif do.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
+        elif dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
             LOG.debug('Request not supported in this OF protocol version')
             return Response(status=501)
         else:
