@@ -34,6 +34,7 @@ from ryu.services.protocols.bgp.info_base.base import AttributeMap
 from ryu.services.protocols.bgp.model import ReceivedRoute
 from ryu.services.protocols.bgp.net_ctrl import NET_CONTROLLER
 from ryu.services.protocols.bgp.rtconf.neighbors import NeighborConfListener
+from ryu.services.protocols.bgp.rtconf.neighbors import CONNECT_MODE_PASSIVE
 from ryu.services.protocols.bgp.signals.emit import BgpSignalBus
 from ryu.services.protocols.bgp.speaker import BgpProtocol
 from ryu.services.protocols.bgp.info_base.ipv4 import Ipv4Path
@@ -73,6 +74,8 @@ from ryu.lib.packet.bgp import BGPPathAttributeLocalPref
 from ryu.lib.packet.bgp import BGPPathAttributeExtendedCommunities
 from ryu.lib.packet.bgp import BGPPathAttributeMpReachNLRI
 from ryu.lib.packet.bgp import BGPPathAttributeMpUnreachNLRI
+from ryu.lib.packet.bgp import BGPPathAttributeCommunities
+from ryu.lib.packet.bgp import BGPPathAttributeMultiExitDisc
 
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_ORIGIN
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_AS_PATH
@@ -408,6 +411,14 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         LOG.debug('set out-filter : %s' % filters)
         self.on_update_out_filter()
 
+    @property
+    def adj_rib_in(self):
+        return self._adj_rib_in
+
+    @property
+    def adj_rib_out(self):
+        return self._adj_rib_out
+
     @property
     def is_route_server_client(self):
         return self._neigh_conf.is_route_server_client
@@ -416,6 +427,10 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
     def check_first_as(self):
         return self._neigh_conf.check_first_as
 
+    @property
+    def connect_mode(self):
+        return self._neigh_conf.connect_mode
+
     @property
     def attribute_maps(self):
         return self._attribute_maps
@@ -529,6 +544,19 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             for af in negotiated_afs:
                 self._fire_route_refresh(af)
 
+    def _on_update_connect_mode(self, mode):
+        if mode is not CONNECT_MODE_PASSIVE and \
+                'peer.connect_loop' not in self._child_thread_map:
+            LOG.debug("start connect loop. (mode: %s)" % mode)
+            self._spawn('peer.connect_loop', self._connect_loop,
+                        self._client_factory)
+        elif mode is CONNECT_MODE_PASSIVE:
+            LOG.debug("stop connect loop. (mode: %s)" % mode)
+            self._stop_child_threads('peer.connect_loop')
+
+    def on_update_connect_mode(self, conf_evt):
+        self._on_update_connect_mode(conf_evt.value)
+
     def _apply_filter(self, filters, path):
         block = False
         blocked_cause = None
@@ -615,11 +643,13 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
 
     def _run(self, client_factory):
         LOG.debug('Started peer %s' % self)
-        # Start sink processing in a separate thread
-        self._spawn('peer.process_outgoing', self._process_outgoing_msg_list)
+        self._client_factory = client_factory
 
-        # Tries actively to establish session.
-        self._connect_loop(client_factory)
+        # Tries actively to establish session if CONNECT_MODE is not PASSIVE
+        self._on_update_connect_mode(self._neigh_conf.connect_mode)
+
+        # Start sink processing
+        self._process_outgoing_msg_list()
 
     def _send_outgoing_route_refresh_msg(self, rr_msg):
         """Sends given message `rr_msg` to peer.
@@ -772,11 +802,6 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         """
         route_family = path.route_family
 
-        if route_family == RF_IPv4_UC and path.nexthop != '0.0.0.0':
-            return path.nexthop
-        if route_family == RF_IPv6_UC and path.nexthop != '::':
-            return path.nexthop
-
         # By default we use BGPS's interface IP with this peer as next_hop.
         if self._neigh_conf.next_hop:
             next_hop = self._neigh_conf.next_hop
@@ -827,7 +852,6 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             unkown_opttrans_attrs = None
             nlri_list = [path.nlri]
 
-            # MP_REACH_NLRI Attribute.
             # By default we use BGPS's interface IP with this peer as next_hop.
             # TODO(PH): change to use protocol's local address.
             # next_hop = self.host_bind_ip
@@ -911,12 +935,15 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             # For eBGP session we can send multi-exit-disc if configured.
             multi_exit_disc = None
             if self.is_ebgp_peer():
-                multi_exit_disc = pathattr_map.get(
-                    BGP_ATTR_TYPE_MULTI_EXIT_DISC)
-                if not multi_exit_disc and self._neigh_conf.multi_exit_disc:
+                if self._neigh_conf.multi_exit_disc:
                     multi_exit_disc = BGPPathAttributeMultiExitDisc(
                         self._neigh_conf.multi_exit_disc
                     )
+                else:
+                    pass
+            if not self.is_ebgp_peer():
+                multi_exit_disc = pathattr_map.get(
+                    BGP_ATTR_TYPE_MULTI_EXIT_DISC)
 
             # LOCAL_PREF Attribute.
             if not self.is_ebgp_peer():
@@ -1448,7 +1475,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 tm.learn_path(w_path)
             else:
                 LOG.debug('prefix : %s is blocked by in-bound filter : %s'
-                          % (msg_nlri, blocked_cause))
+                          % (nlri_str, blocked_cause))
 
     def _extract_and_handle_mpbgp_new_paths(self, update_msg):
         """Extracts new paths advertised in the given update message's
